import sys
import hou

from streets import *
from lots import *

from controlVariables import *

class geometry:

	def createGeometry(self):

		geo = hou.node('/obj').createNode('geo')
		controlVariables.geo = geo

		#delete the file node
		for n in geo.children():
			n.destroy()

		tra = geo.createNode('trace')
		if controlVariables.loadMap != None:
			tra = self.carregarMapaCiutatReal(tra)
		else:
			tra = self.carregarImatgeCOP2(tra)
		tra.parm('doresample').set(1)
		tra.parm('step').set(15)

		if (controlVariables.districtMap!=None):
			tra = self.esborrarPrimitivaExterior(geo,tra)

		#delete too little primitives AND call the streets module (if needed)
		tra = self.esborrarPrimitivesPetites(geo,tra)
		
		tra2 = self.triarPrimitivesVisibles(geo,tra)
		
		#scale the whole terrain AND call the lots module
		tra2 = self.escalar(geo,tra,tra2)

		#the floor texture
		if controlVariables.floor != None:
			tra2 = self.posarTextura(geo,tra2)

	
		mergefinal = geo.createNode('object_merge')
		mergefinal.parm('numobj').set(2)
		mergefinal.parm('objpath1').set('/obj/geo1/citySubnet/scale')
		mergefinal.parm('objpath2').set('/obj/geo1/citySubnet/scale2')
		mergefinal.setDisplayFlag(True)
		
		geo.setDisplayFlag(True)
		
		for n in geo.children():
			n.moveToGoodPosition()

		for n in hou.node('/obj').children():
			n.moveToGoodPosition()



	def carregarMapaCiutatReal(self,tra):
		tra.parm('file').set(controlVariables.loadMap)#use the map
		tra.parm('usecop1').set(0)
		return tra


	def carregarImatgeCOP2(self,tra):
		tra.parm('coppath').set('/img/img1/final')#use COP2
		tra.parm('usecop1').set(1)
		return tra


	def esborrarPrimitivaExterior(self,geo,tra):
		dele = geo.createNode('delete')
		dele.setFirstInput(tra)
		dele.parm('pattern').set('0')
		return dele


	def esborrarPrimitivesPetites(self,geo,tra):
		#measure primitives area
		mea3 = geo.createNode('measure') 
		mea3.parm('override').set(1) #override name
		mea3.parm('type').set(1) #compute the areas
		mea3.parm('attribname').set('areaLots')
		mea3.setFirstInput(tra)

		attpr2 = geo.createNode('attribpromote')
		attpr2.setName('avgarea')
		attpr2.setFirstInput(mea3)
		attpr2.parm('inname').set('areaLots') #original name
		attpr2.parm('inclass').set(1) #original class: primitive
		attpr2.parm('outclass').set(0) #new class: detail
		attpr2.parm('method').set(2) #promotion method: average
		attpr2.parm('useoutname').set(1) #change new name
		attpr2.parm('outname').set('avgarea')

		#measure primitives area
		mea = geo.createNode('measure')
		mea.parm('override').set(1) #override name
		mea.parm('type').set(1) #compute the areas
		if (controlVariables.pattern == 'grid') | (controlVariables.pattern == 'radial'):
			s = streets()
			mea.setFirstInput(s.streets(attpr2)) #call the streets module
		else:
			mea.setFirstInput(tra)

		attpr = geo.createNode('attribpromote')
		attpr.setFirstInput(mea)
		attpr.parm('inname').set('area') #original name
		attpr.parm('inclass').set(1) #original class: primitive
		attpr.parm('outclass').set(0) #new class: detail
		attpr.parm('method').set(2) #promotion method: average
		attpr.parm('useoutname').set(1) #change new name
		attpr.parm('outname').set('totsize')

		#delete too little primitives
		del5 = geo.createNode('delete')
		del5.setName('del')
		del5.setFirstInput(mea)
		del5.parm('groupop').set(2) #delete by expression
		if (controlVariables.districtMap!=None) | (controlVariables.pattern == 'grid'):
			del5.parm('filter').setExpression("$AREA < 0.7 * detail(\"../attribpromote1\",\"totsize\",0)")
		else:
			del5.parm('filter').setExpression("$AREA < 0.2 * detail(\"../attribpromote1\",\"totsize\",0)")

		mea2 = geo.createNode('measure')
		mea2.setFirstInput(del5)
		mea2.parm('override').set(1) #override name
		mea2.parm('type').set(0)  #compute perimeters
		mea2.parm('attribname').set('perimet')

		attpr2 = geo.createNode('attribpromote')
		attpr2.setFirstInput(mea2)
		attpr2.setName('avgperim')
		attpr2.parm('inname').set('perimet')
		attpr2.parm('inclass').set(1) #primitive
		attpr2.parm('outclass').set(0) #detail
		attpr2.parm('method').set(2) #average
		attpr2.parm('useoutname').set(1) #change new name
		attpr2.parm('outname').set('avgperim')	
		attpr2.parm('deletein').set(0) #don't delete the original
		
		return attpr2
	
	
	def triarPrimitivesVisibles(self,geo,tra):
		del7 = geo.createNode('delete')
		del7.setName('visibles')
		del7.setFirstInput(tra)
		del7.parm('negate').set(1) #delete non-selected
		del7.parm('groupop').set(1) #delete by range
		del7.parm('select1').set(1)
		del7.parm('select2').set(1)
		del7.parm('rangestart').set(0)
		del7.parm('rangeend').setExpression('$N')
		
		return del7

	def escalar(self,geo,tra,tra2):
		#scale the whole terrain
		scale2 = geo.createNode('xform')
		scale2.setName('scale2')
		scale2.setFirstInput(tra)
		scale2.parm('sx').set(1)
		scale2.parm('sy').set(1)
		scale2.parm('sz').set(1)
	
		#scale the visible lots
		scale = geo.createNode('xform')
		scale.setName('scale')
		l = lots()
		scale.setFirstInput(l.lots(tra2)) #call the lots module
		scale.parm('sx').set(1)
		scale.parm('sy').set(1)
		scale.parm('sz').set(1)
	
		return scale


	def posarTextura(self,geo,scale):
		grid = geo.createNode('grid')
		grid.parm('orient').set(0) #xy pane
		grid.parm('sizex').set(1)
		grid.parm('sizex').set(1)

		shade = geo.createNode('uvquickshade')
		shade.setFirstInput(grid)
		shade.parm('texture').set(controlVariables.floor)
		shade.parm('texture_axis').set(2) #z axis

		matchsize = geo.createNode('matchsize')
		matchsize.setFirstInput(shade)
		matchsize.setInput(1,scale)

		xform = geo.createNode('xform')
		xform.setFirstInput(matchsize)
		xform.parm('tz').set(-0.004)

		finalmerge = geo.createNode('merge')
		finalmerge.setFirstInput(scale)
		finalmerge.setInput(1,xform)