import sys
import hou

from buildings import *

from controlVariables import *

class lots:

	def lots(self,attpr2):
		
		geo = controlVariables.geo

		#delete all of the primitives but one at the time
		del6 = geo.createNode('delete')
		del6.setFirstInput(attpr2)
		del6.parm('affectnumber').set(0)
		del6.parm('group').set(r'`stamp("../finalcopy","ccy",0)`')
		del6.parm('negate').set(1)
		
		mea3 = geo.createNode('measure')
		mea3.setFirstInput(del6)
		mea3.parm('override').set(1)
		mea3.parm('attribname').set('perim')
		mea3.parm('type').set('perimeter') #perimeter
		mea3.setName('measurePerim')

		
		attp = self.crearPrimitivaInterior(geo,mea3)
		
		copy2 = geo.createNode('copy')
		copy2.setName('finalcopy')
		copy2.parm('ncy').setExpression('nprims("../del")') #number of copies
		copy2.parm('stamp').set(1)
		copy2.parm('param1').set('ccy') 
		copy2.parm('val1').setExpression("$CY") #the value is the copy number
		controlVariables.numPrims = copy2.parm('ncy').eval()
		#print(controlVariables.numPrims)
		
		
		#we want to have for each segment a random amount of extrusion
		#extract segments
		car = geo.createNode('carve')
		car.setFirstInput(attp)
		car.parm('secondu').set(1)
		car.parm('domainu2').set(1)
		car.parm('domainu1').setExpression('3*stamp("../copy2","seg",0)/npoints("../point1")')
		car.parm('domainu2').setExpression('3*(stamp("../copy2","seg",0)+1)/npoints("../point1")')
		
		#move along the normal
		pea2 = geo.createNode('peak')
		pea2.setFirstInput(car)
		pea2.parm('updatenmls').set(0) #don't recompute point normals
		pea2.parm('dist').setExpression('1.2*sqrt(rand(stamp("../copy2","seg",0)*prim("../attribpromote2",0,"distP",0)/point("../carve1",0,"dist",0)))')

		ski2 = self.fusionar(geo,car,pea2)

		copy3 = geo.createNode('copy')
		copy3.parm('ncy').setExpression('npoints("../point1")/3') #as many copies as number of points
		copy3.parm('stamp').set(1)
		copy3.parm('param1').set('seg') 
		copy3.parm('val1').setExpression("$CY") #the value is the copy number
		copy2.setFirstInput(copy3)

		b = buildings()
		aux = b.buildings(ski2)
		copy3.setFirstInput(aux)
		
		return copy2
	
	
	def dividirParceles(self,geo,mea3):
	
		res3 = geo.createNode('resample')
		res3.setFirstInput(mea3)
		res3.parm('dolength').set(0)
		res3.parm('dosegs').set(1)
		res3.parm('segs').setExpression('8 * prim("../measurePerim",0,"perim",0) / detail("../avgperim","avgperim",0)')

		#divide in little segments
		res2 = geo.createNode('resample')
		res2.setFirstInput(res3)
		res2.parm('edge').set(1)
		res2.parm('dolength').set(0)
		res2.parm('dosegs').set(1)
		res2.parm('segs').set(3)
		
		poi = self.roundAngles(geo,res2)
		
		return poi
	
	
	def roundAngles(self,geo,res2):
		cir = geo.createNode('circle')
		cir.parm('type').set(2) #NURBS curve
		cir.parm('divs').setExpression('npoints("../resample2")')
	
		#force the circle to have the shape of resample
		poi = geo.createNode('point')
		poi.setFirstInput(cir)
		poi.setInput(1,res2)
		poi.parm('tx').setExpression('$TX2') #it takes the position of the resample
		poi.parm('ty').setExpression('$TY2')
		poi.parm('tz').setExpression('$TZ2')
	
		return poi
		
		
	def crearPrimitivaInterior(self,geo,mea3):
		#the inner primitive
		poli3 = geo.createNode('polyextrude')
		poli3.setName('buildingSize')
		poli3.parm('lsx').set(controlVariables.lotsWidth) #scale
		poli3.parm('lsy').set(controlVariables.lotsWidth)
		poli3.parm('outputside').set(0) #just output front
		
		ray = geo.createNode('ray')
		ray.setInput(1,poli3)
		ray.parm('entity').set(1) #points
		ray.parm('method').set(0) #minimum distance
		ray.parm('putdist').set(1) #point intersection distance

		#the new normal
		poi2 = geo.createNode('point')
		poi2.setInput(1,ray)
		poi2.parm('donml').set(1) #add normal
		poi2.parm('nx').setExpression("$TX2-$TX") #the current point of the second input minus
		poi2.parm('ny').setExpression("$TY2-$TY")  #the current point of the first input
		poi2.parm('nz').setExpression("$TZ2-$TZ")
	
		poi = self.dividirParceles(geo,mea3)
		poli3.setFirstInput(poi)
		ray.setFirstInput(poi)
		poi2.setFirstInput(poi)
	
		#the distance between corresponding points of the inner and outer primitives
		#is an attribute of the inner primitive and we want to transfer it to the outer one
		att = geo.createNode('attribtransfer')
		att.setFirstInput(poi2)
		att.setInput(1,ray)
		att.parm('srcgrouptype').set(1) #points
		att.parm('dstgrouptype').set(1) #points
		att.parm('pointattriblist').set('dist')
	
		attp = geo.createNode('attribpromote')
		attp.setFirstInput(att)
		attp.parm('inname').set('dist')
		attp.parm('outclass').set(1)
		attp.parm('method').set(1)
		attp.parm('useoutname').set(1)
		attp.parm('outname').set('distP')
		attp.parm('deletein').set(0)
	
		return attp

	
	def fusionar(self,geo,car,pea2):
		#merge the inner and the outer primitives
		mer2 = geo.createNode('merge')
		mer2.setFirstInput(car)
		mer2.setInput(1,pea2)
		
		#join the segments
		ski2 = geo.createNode('skin')
		ski2.setFirstInput(mer2)

		return ski2